#
# Author:: Frederic Nowak (<frederic.nowak@alphard.io>)
# Cookbook:: alphard-chef-deploy
# Definition:: install
#
# Copyright:: 2017, Hydra Technologies, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
define :install, organization: nil do
  # Defines includes
  include_recipe 'java'

  # Defines variables
  name = params[:name]
  organization = params[:organization]
  installer = proc do |configurations|
    # Defines variables
    configuration = configurations[:configuration]
    version_directory = configuration[:version_directory]
    current_directory = configuration[:current_directory]

    # Defines and creates jvm directories
    current_temporary_directory = "#{current_directory}/#{configuration[:temporary_directory]}"
    current_log_directory = "#{current_directory}/#{configuration[:log_directory]}"
    current_library_directory = "#{current_directory}/#{configuration[:library_directory]}"
    current_configuration_directory = "#{current_directory}/#{configuration[:configuration_directory]}"
    current_binary_directory = "#{current_directory}/#{configuration[:binary_directory]}"
    current_execution_path = "#{current_binary_directory}/#{configuration[:platform]}"
    current_executable = "#{current_execution_path}/#{configuration[:executable_name]}"

    version_library_directory = "#{version_directory}/#{configuration[:library_directory]}"
    version_configuration_directory = "#{version_directory}/#{configuration[:configuration_directory]}"
    version_binary_directory = "#{version_directory}/#{configuration[:binary_directory]}"

    [current_temporary_directory, current_log_directory, version_directory].each do |d|
      directory d do
        owner configuration[:user]
        group configuration[:group]
        mode '0755'
        action :create
      end
    end

    # Updates configurations
    configuration = configuration.merge(library_path: current_library_directory,
                                        configuration_path: current_configuration_directory,
                                        binary_path: current_binary_directory,
                                        temporary_path: current_temporary_directory,
                                        log_path: current_log_directory)
    configurations = configurations.merge(configuration: configuration)

    # Defines Linux service
    linux_service = "#{cookbook_name}-#{recipe_name}"
    linux_service_pid_path = "/var/run/#{linux_service}.pid"
    linux_service_script = "/etc/init.d/#{linux_service}"
    linux_service_script_exists = ::File.exist?(linux_service_script)

    # Stops service
    execute "stop service #{linux_service}" do
      command "#{linux_service_script} stop"
      only_if { linux_service_script_exists }
    end

    # Installs init.d script
    linux_service_template = configuration[:script_template]
    linux_service_class_path = "#{current_library_directory}/classes"
    linux_service_class_path += ":#{current_library_directory}/jars/*"
    linux_service_class_path += ":#{current_configuration_directory}"
    linux_service_library_path = "#{current_library_directory}/natives/#{configuration[:platform]}"
    linux_service_java_options = "-Dalphard.service.version=#{configuration[:version]}"
    linux_service_java_options += " -cp #{linux_service_class_path} #{configuration[:java_options]}"
    linux_service_java_options += " -Djava.library.path=#{linux_service_library_path}"

    template linux_service_script do
      source linux_service_template
      mode '0755'
      owner 'root'
      group 'root'
      cookbook 'alphard-chef-deploy'
      variables configurations.merge(pid_path: linux_service_pid_path,
                                     name: linux_service,
                                     description: configuration[:description],
                                     java_options: linux_service_java_options,
                                     main_class: configuration[:main_class],
                                     main_arguments: configuration[:main_arguments])
      action :create
    end

    # Enables service
    execute "enable service #{linux_service}" do
      command "update-rc.d #{linux_service} defaults"
      only_if { !linux_service_script_exists }
    end

    # Installs the package
    if configuration[:packaging] == 'application'
      install_application configuration[:name] do
        configurations configurations
        directory version_directory
      end
    elsif configuration[:packaging] == 'service'
      install_service configuration[:name] do
        configurations configurations
        directory version_directory
        library_directory version_library_directory
        configuration_directory version_configuration_directory
        binary_directory version_binary_directory
      end
    else
      puts "invalid packaging type #{configuration[:packaging]}, should be 'application' or 'service'"
      return
    end

    # Links current directories to version directories
    { current_library_directory => version_library_directory,
      current_configuration_directory => version_configuration_directory,
      current_binary_directory => version_binary_directory }.each do |target_file, to|
      link target_file do
        owner user
        group group
        target_file target_file
        to to
        action :create
      end
    end

    # Grants execution permission on executable
    execute "chmod +x #{current_executable}" do
      command "chmod +x #{current_executable}"
      user user
      group group
    end

    # Starts
    execute "start service #{linux_service}" do
      command "#{linux_service_script} start"
    end
  end

  # Configures and installs
  configure name do
    organization organization
    installer installer
  end
end
